//+----------------------------------------------------------------------------
//| Stochastic Slope Histo.mq4                                                
//| Copyright  2010, Xaphod                                                  
//| http://forexwhiz.appspot.com                                          
//| Description: Indicator that shows the slope stochastic main line is above  
//|              or below a set value with a dual colored histogram.
//|              Since we have different units on the x and y axis, the term  
//|              slope is used here instead of angle.                         
//|              The threshold value for the slope is a value between 0.0 and
//|              1.0. As the slope you see varies with the height of the chart, 
//|              you will have to empirically decide what value works for you.
//| Change Log:                                                               
//|   2010-03-22, Xaphod. Implementation. v1.00 Released   
//|   2010-03-25, Xaphod. Added indicator line width properties. v1.02
//+----------------------------------------------------------------------------
#property copyright "Copyright  2010, Xaphod"
#property link      "http://forexwhiz.appspot.com"

//---- Indicator properties
#property indicator_separate_window
#property indicator_minimum -1
#property indicator_maximum 1
#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 DarkSeaGreen
#property indicator_width1  2
#property indicator_width2  2

//---- Input parameters
extern string    SettingsInfo="<< %K Period, %D Perios, Slowing >>";
extern int       KPeriod=8;
extern int       DPeriod=3;
extern int       Slowing=3;
extern string    PriceInfo="<< 0=High/Low, 1=Close/Close >>";
extern int       PriceField=0;
extern string    MAInfo="<< SMA=0, EMA=1, SMMA=2, LWMA=3 >>";
extern int       MAMethod=0;
extern string    SlopeThreshholdInfo="<< Slope Threshold=0.0 to 1.0 >>";
extern double    SlopeThreshold=0.1;
extern string    BarHeightInfo="<< Set the histo bar Height: 1-4 >>";
extern double    BarHeight=2;

//---- Indicator buffers
double dExtMapBuffer1[];
double dExtMapBuffer2[];

//---- Indicator name & version
string sIndicatorName;
string sVersion = "v1.02";


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
  //---- indicators
  SetIndexStyle(0,DRAW_HISTOGRAM,EMPTY);
  SetIndexBuffer(0,dExtMapBuffer1);
  SetIndexStyle(1,DRAW_HISTOGRAM,EMPTY);
  SetIndexBuffer(1,dExtMapBuffer2);
  
  //---- Set indicator name  
  sIndicatorName = "[Stoch(" + KPeriod + "," + DPeriod + "," + Slowing+") " + "Slope Histo " + sVersion + "]";
  IndicatorShortName(sIndicatorName);
  
  // Validate parameters
  if (SlopeThreshold>1) SlopeThreshold=1;
  if (SlopeThreshold<0) SlopeThreshold=0;
  if (BarHeight<1) BarHeight=1;
  if (BarHeight>4) BarHeight=4;
    
  return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() {
  //----   
  return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
  double dStocMain0;    // Value Main (%K Period) on 0 bar
  double dStocMain1;    // Value Main (%K Period) on 1 bar
  double dSlope;        // Slope from 1 bar to 0 bar
  int    iNewBars;      // New bars to be processed
  int    iCountedBars=IndicatorCounted();
  
  // Get new bars
  if(iCountedBars<0) return(-1);  
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  

  //---- main loop  
  for(int i=iNewBars-2; i>=0; i--) {  
    // Calc stochastic values and slope 
    dStocMain0 = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,MAMethod,PriceField,MODE_MAIN,  i); // 0 bar
    dStocMain1 = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,MAMethod,PriceField,MODE_MAIN,  i+1); // 1st bar
    dSlope=((dStocMain0-dStocMain1)/100);
    
    // Draw histogram
    if (dSlope>SlopeThreshold || dSlope<-SlopeThreshold )
      dExtMapBuffer2[i]=dSlope*BarHeight;
    else
      dExtMapBuffer1[i]=dSlope*BarHeight;
  }
  
  return(0);
}
//+------------------------------------------------------------------+