//+-------------------------------------------------------------------+
//|	Fisher_Transform_Histogram.mq4
//|	Based on Symphonie_Market_Emotion_Indicator Indicator
//| !!! this file is created with Tab size 4 and no space indentation
//+-------------------------------------------------------------------+
#property copyright "Comer"
#property link      ""

#property indicator_separate_window

#property indicator_buffers 2

#property indicator_color1	DodgerBlue
#property indicator_style1	STYLE_SOLID
#property indicator_width1	4

#property indicator_color2	Red
#property indicator_style2	STYLE_SOLID
#property indicator_width2	4

//---- input parameters
extern int	Length		= 12;
extern bool Collapse	= false;
/*extern*/ int Price_smoothing_period	= 3;
/*extern*/ int Fisher_smoothing_period	= 5;

//---- buffers
double SignalUp[];   
double SignalDown[];   

#define IDX_PRICE	0
#define IDX_FISHER	1

#define VALUE_COLLAPSE	1.0
#define VALUE_SPREAD	-1.0

double	normalizedPrice = 0.0;
double	tmpPrice = 0.0;
double	fisher = 0.0;
double	tmpFisher = 0.0;
int		lastBar = -1;
double	alpha[2];
double	beta[2];
double	valueDown;

int BarNumber;
int BarIndex;

int init() {
	SetIndexStyle ( 0, DRAW_HISTOGRAM	);
	SetIndexBuffer( 0, SignalUp			);
	SetIndexStyle ( 1, DRAW_HISTOGRAM	);
	SetIndexBuffer( 1, SignalDown		);
	
	alpha[ IDX_PRICE	] = alpha( Price_smoothing_period );
	alpha[ IDX_FISHER	] = alpha( Fisher_smoothing_period );
	beta [ IDX_PRICE	] = beta( alpha[ IDX_PRICE ] );
	beta [ IDX_FISHER	] = beta( alpha[ IDX_FISHER ] );

	if( Collapse )
		valueDown = VALUE_COLLAPSE;
	else
		valueDown = VALUE_SPREAD;

	return(0);
}

double alpha( int period ) {
	return (2.0 / (period + 1.0));
}

double beta( double alpha ) {
	return (1.0 - alpha);
}

int start() {
	BarNumber = IndicatorCounted();	// [0 .. Bars]
   
	for( BarIndex = Bars - BarNumber; BarIndex > 0; ) {
		BarIndex--;		// [Bars-1 .. 0]
		processBar();
		BarNumber++;
	}
   
	return(0);
}

void processBar() {
	double	hh;
	double	ll;
	double	price;

	// protection from the same bar update - use the same initial value as the previous update of the same bar
	if( BarNumber != lastBar ) { // if it was the same bar 'fisher' and 'normalizedPrice' would stay the same as the last calculation
		normalizedPrice = tmpPrice;
		fisher = tmpFisher;
	}

	price = (High[ BarIndex ] + Low[ BarIndex ]) / 2.0;

	hh = High[ iHighest(NULL, 0, MODE_HIGH, Length, BarIndex) ];
	ll = Low [ iLowest (NULL, 0, MODE_LOW,  Length, BarIndex) ];

	if( hh == ll )
		hh = ll + Point; // to prevent Math errors

	double nprice = 2.0*( (price-ll)/(hh-ll) - 0.5 );
	if( BarNumber < 2 )
		normalizedPrice = nprice;
	else
		normalizedPrice = alpha[ IDX_PRICE ]*nprice + beta[ IDX_PRICE ]*normalizedPrice;
	tmpPrice = MathMin( MathMax(normalizedPrice, -0.9999), 0.9999 );

	double fish = 0.5*MathLog( (1.0+tmpPrice)/(1.0-tmpPrice));
	if( BarNumber < 2 )
		tmpFisher = fish;
	else
		tmpFisher = alpha[ IDX_FISHER ]*fish + beta[ IDX_FISHER ]*fisher;

	if( tmpFisher > 0 ) {
		SignalUp[ BarIndex ] = 1;
		SignalDown[ BarIndex ] = 0;
	}
	else {
		SignalUp[ BarIndex ] = 0;
		SignalDown[ BarIndex ] = valueDown;
	}
		
	lastBar = BarNumber;
}