//+-----------------------------------------------------------------------+
//|                                 Reference                             |
//|  Reference   http://www.forexfactory.com/maximumguita                 |
//|      http://www.forexfactory.com/showthread.php?p=8897483#post8897483 |
//+-----------------------------------------------------------------------+
#property copyright "http://www.forexfactory.com/lukeb"
#property link      "http://www.forexfactory.com/showthread.php?p=8913400#post8913400"
#property strict
//
#property indicator_chart_window
#property indicator_buffers 2
//
enum signalMode
{
   onPA,
   onBar
};
//
input   int    normalAvgPeriod  = 15;              // 15, normal averaging period for the WPR indicator
input   double wprUpper         = -27;             // -30 is the usual upper WPR line, this is the upper signal level
input   double wprLower         = -73;             // -70 is the usual lower WPR line, this is the lower signal level
input   double resetSignal      = 20;              // After the signal retraces this amount, allow another same direction signal.
input   signalMode  signaling   = onPA;           // Signal on first Price Action cross, or on Bar Completion.
input   color  DownArrowClr     = clrRed;          // Color the Down Arrow
input   color  UpArrowClr       = clrLimeGreen;    // Color the Up Arrow
input   string MailSubject      = "Market Entry";  // Subject Line for Mail
input   bool   SoundAlarm       = TRUE;            // True == Alert on Terminal
input   bool   EmailAlarm       = FALSE;           // True == Send an Email
input   bool   SendText         = FALSE;           // True == Send a Text
input   bool   TagWithPrice     = TRUE;           // True == Put Price Tags on the chart
input   int    TagWidth         = 2;               // Size of the Price Label
input   color  TagSellClr       = clrRed;          // Color of the Sell Price Label
input   color  TagBuyClr        = clrYellowGreen;  // Color of the Buy Price Label
input   bool   ScrnDebug        = FALSE;           // Display on-chart Debug Info
input   bool   PrntDebug        = FALSE;           // Print Debug Statements are active
//
int oldestBarToProcess = 10000;
const int stepBack     = 12;
const int BARZERO      = 0;
//
double  sellArrowBuff[];
double  buyArrowBuff[];
bool    sellSignaled       = false;
bool    buySignaled        = false;
int     priceLabelCount    = 0;
const string wprPriceLabel = "wprPiceLbl";
//
const string oneDebugDisplayName    = "one1wprDebugDisplay";
const string twoDebugDisplayName    = "two2wprDebugDisplay";
const string threeDebugDisplayName  = "three3wprDebugDisplay";
const int    DISPLAYWINDOWZERO      = 0;
const int    CURRENTCHART           = 0;
//
double workWPRUpper    = wprUpper;
double workWPRLower    = wprLower;
double workResetSignal = resetSignal;
//
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
   IndicatorBuffers(2);
   SetIndexStyle(0, DRAW_ARROW,0,1,DownArrowClr);
   SetIndexArrow(0, 234);
   SetIndexEmptyValue(0, NULL);
   SetIndexStyle(1, DRAW_ARROW,0,1,UpArrowClr);
   SetIndexArrow(1, 233);
   SetIndexEmptyValue(1, NULL);
   SetIndexBuffer(0, sellArrowBuff);
   SetIndexBuffer(1, buyArrowBuff);
   if (workWPRUpper < workWPRLower)  // invalid, use what the wpr indicator defaults to
   {
      workWPRUpper = -30;
      workWPRLower = -70;
   }
   double upperToLowerGap = MathAbs(workWPRLower-workWPRUpper);
   if ( upperToLowerGap < workResetSignal )
   {
      workResetSignal = (upperToLowerGap/2);
   }
   return (INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//|    DeInit Function                                               |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
    DeletePriceLabels();
    ObjectDelete(oneDebugDisplayName);
    ObjectDelete(twoDebugDisplayName);
    ObjectDelete(threeDebugDisplayName);
}
void DeletePriceLabels(void)
{
   for ( int i=0; i < priceLabelCount; i++ )
   {
      ObjectDelete( wprPriceLabel+IntegerToString(i) );
   }
   priceLabelCount = 0;
}
bool debug  = ScrnDebug;
bool debug1 = PrntDebug;
//+------------------------------------------------------------------+
//|  Custom indicator iteration function.                            |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total, const int prev_calculated, const datetime &time[], const double &open[], const double &high[],
                const double &low[], const double &close[], const long& tick_volume[], const long& volume[], const int& spread[])
{
   static int onCalcCallCount = 0;
   onCalcCallCount++;
   static datetime buySigTime = 0, sellSigTime = 0;
   int processStart;
   // Figure out the bar to begin processing with
   if ( prev_calculated < 1 )             // First call, prev_calculated == 0, initialize
   {
      if (rates_total <= stepBack)
      {
         return(0);  // Nothing processed, return zero until there are enough bars
      }
      if ( (oldestBarToProcess+stepBack) > rates_total)
      {
         oldestBarToProcess = (rates_total - stepBack);
      }
      SetIndexDrawBegin(0, Bars - (oldestBarToProcess + stepBack) );
      SetIndexDrawBegin(1, Bars - (oldestBarToProcess + stepBack) );
      DeletePriceLabels();
      processStart = oldestBarToProcess;
      if (debug1)
         Print( "On CalcCalls: ",IntegerToString(onCalcCallCount),", prev_calculated: ", IntegerToString(prev_calculated),", rates_total: ",IntegerToString(rates_total),", OldestBar: ",IntegerToString(oldestBarToProcess));
   } else
   {
      processStart = rates_total - prev_calculated;
      if (debug1)
         Print( "On CalcCalls: ",IntegerToString(onCalcCallCount),"prev_calculated: ", IntegerToString(prev_calculated),", rates_total: ",IntegerToString(rates_total),", OldestBar: ",IntegerToString(processStart));
   }
   //
   string dispString;
   //
   for (int barToProcess = processStart; barToProcess >= 0; barToProcess--)
   {
      static int forLoopCallCount = 0;
      forLoopCallCount++;
      if (debug1 && (barToProcess == 0) )
         Print( "For Loop Call count: ",IntegerToString(forLoopCallCount),", barToProcess",IntegerToString(barToProcess),", prev_calculated: ", IntegerToString(prev_calculated),", rates_total: ",IntegerToString(rates_total),", OldestBar: ",IntegerToString(processStart));
      // Calculate 10 bar ATR
      double tenBarATR = GetATR( barToProcess, stepBack );   // stepback or 10?
      int wprAvgPeriod = GetWPRAvrgPeriod( barToProcess, tenBarATR );
      double wprValue  = -MathAbs(iWPR(NULL, 0, wprAvgPeriod, barToProcess));
      //
      // sellArrowBuff[barToProcess] = 0;
      // buyArrowBuff[barToProcess]  = 0;
      //
      //----- Set the Buffer to the Arrow Value -------------
      if ( (wprValue < workWPRLower) && (false == sellSignaled) )  // let the arrow flicker on and off on Bar 0
      {  // Sells are triggered in the directioon of the indicator, the indicator is Down, more negative than workWPRLower
         sellArrowBuff[barToProcess] = GetArrowPrice(barToProcess, OP_SELL, tenBarATR);   // Set the arrow above the bar
         if ( (signaling == onPA) && ((false == sellSignaled)) )
         {
            sellSignaled = SetSignal(OP_SELL, barToProcess);
            sellSigTime = Time[barToProcess];
         }
      } else if ( sellSigTime != Time[barToProcess] )
      {
         sellArrowBuff[barToProcess] = NULL;
      }
      if ( (wprValue > workWPRUpper) && (false == buySignaled) )  // let the arrow flicker on and off on Bar 0
      {  // Buys are triggered in the directioon of the indicator, the indicator is up, less negative than workWPRUpper
         buyArrowBuff[barToProcess] =  GetArrowPrice(barToProcess, OP_BUY, tenBarATR); // Set the arrow below the bar
         if ( (signaling == onPA) && ((false == buySignaled)) )
         {
            buySignaled = SetSignal(OP_BUY, barToProcess);
            buySigTime = Time[barToProcess];
         }
      } else if ( buySigTime != Time[barToProcess] )
      {
         buyArrowBuff[barToProcess] = NULL;
      }
      //
      //------ Signal is real, leave it marked, turn of new marks until Re-Signalling is allowed
      if ( (signaling == onBar) && (sellArrowBuff[barToProcess+1] > Point) && (false == sellSignaled) )  // test for a value greater than zero and alerts not yet sent
      {
         sellSignaled = SetSignal(OP_SELL, barToProcess);
         sellSigTime = Time[barToProcess];
      }
      if ( (signaling == onBar) && (buyArrowBuff[barToProcess+1] > Point) && (false == buySignaled) )  // test for a value greater than zero and alerts not yet sent
      {
         buySignaled = SetSignal(OP_BUY, barToProcess);
         buySigTime = Time[barToProcess];
      }
      //
      //------ Re-enable Signaling
      if ( (sellSignaled == true) && (wprValue > workWPRLower) && ((wprValue - workWPRLower) > workResetSignal) )  // reset to allow sells to be signaled again. (-50) - (-70) = 20.
      {
         sellSignaled = false;
      }
      if ( (buySignaled == true) && (wprValue < workWPRUpper) && ((workWPRUpper - wprValue) > workResetSignal) )  //  reset to allow buys to be signaled again.  (-30) - (-60) = 30.
      {
         buySignaled = false;
      }
      if (debug)
      {
         dispString = StringConcatenate("FLCalls: ",IntegerToString(forLoopCallCount),", wprVal: ",DoubleToStr(wprValue,2),", Up: ",DoubleToStr(workWPRUpper,2),", Low: ",DoubleToStr(workWPRLower,2));
         DrawLabel( oneDebugDisplayName, dispString, 20, 50, 14,"Arial Narrow",clrRed);
         dispString = StringConcatenate("PLC: ",IntegerToString(priceLabelCount),", Buy: ",(buySignaled)?"1":"0",", Sell: ",(sellSignaled)?"1":"0",", BBuff: ",DoubleToStr(buyArrowBuff[barToProcess],Digits),", SBuff: ",DoubleToStr(sellArrowBuff[barToProcess],Digits));
         DrawLabel( twoDebugDisplayName, dispString, 20, 70, 14,"Arial Narrow",clrRed);
         dispString = StringConcatenate("Mode: ",(signaling==onBar)?"onBar":"onPA",", RstVal: ",DoubleToStr(resetSignal,2),", Buy Sigs: ",CountSignals(buyArrowBuff),", Sell Sigs: ",CountSignals(sellArrowBuff));
         DrawLabel( threeDebugDisplayName, dispString, 20, 90, 14,"Arial Narrow",clrRed);
      }
   }
   return(rates_total);
}
//+---------------------------------------------------------------------+
//| Count Arrows                                                        |
//+---------------------------------------------------------------------+
string CountSignals( double& signalBuffer[] )
{
   int count = 0;
   for ( int i = 0; i < oldestBarToProcess; i++ )
   {
      if ( signalBuffer[i]>Point )
         count++;
   }
   return( IntegerToString(count) );
}
//+---------------------------------------------------------------------+
//| Get the proce for displaying the Signal Arrow                       |
//+---------------------------------------------------------------------+
double GetArrowPrice( int barToProcess, int operation, double tenBarATR)
{
   double arrowPrice = Bid;
   switch (operation)
   {
      case OP_SELL:
         arrowPrice = ( High[barToProcess] + (tenBarATR / 2.0) );  // Set the arrow above the bar
         break;
      case OP_BUY:
         arrowPrice = ( Low[barToProcess] - (tenBarATR / 2.0) );  // Set the arrow below the bar
         break;
      default:
         Print("Error in GetArrowPrice, Invalid Operation = ",IntegerToString(operation));
         break;
   }
   return( arrowPrice );
}
//+---------------------------------------------------------------------+
//| Process signals when set in the indicator                           |
//+---------------------------------------------------------------------+
bool SetSignal( int operation, int barToProcess)
{
   switch (operation)
   {
      case OP_SELL:
         if ( (signaling == onBar) )
         {
            sellArrowBuff[barToProcess] = 0; // turn off signal on the current bar, leave on on previous bar
         }
         sellSignaled = true;
         if ( TagWithPrice )
            DrawPriceTag( OP_SELL, barToProcess );
         if ( BARZERO == barToProcess )
         {
            PostSellAlerts(Open[barToProcess]);
         }
         break;
      case OP_BUY:
         if ( (signaling == onBar) )
         {
            buyArrowBuff[barToProcess] = 0; // turn off signal on the current bar, leave on on previous bar
         }
         buySignaled = true;
         if ( TagWithPrice )
            DrawPriceTag( OP_BUY, barToProcess );
         if ( BARZERO == barToProcess )
         {
            PostBuyAlerts(Open[barToProcess]);
         }
         break;
      default:
         Print("Error in SetSignal, Invalid Operation = ",IntegerToString(operation));
         break;
   }
   return( true );
}
//+---------------------------------------------------------------------+
//| Get the Averaging Period for the Williams Percent Ratio Calculation |
//+---------------------------------------------------------------------+
int GetWPRAvrgPeriod( int barToProcess, double tenBarATR )
{
   int wprAvgPeriod = normalAvgPeriod;
   // Identiy a bar with a Gap or return NotFound (-1)
   // int gapBar = GetAGapBar( barToProcess, stepBack, 2.0*tenBarATR );  // stepback or 10?
   // Identiy a quick Big Move bar return NotFound (-1)
   // int bigMoveBar = GetPostBigMoveBar( bartoprocess, stepBack, 4.6 * tenBarATR);  // stepback or 10?
   if ( -1 < GetPostBigMoveBar( barToProcess, stepBack, 4.6 * tenBarATR) )
   {
      wprAvgPeriod = 4;
   } else if ( -1 < GetAGapBar( barToProcess, stepBack, 2.0*tenBarATR ) )
   {
      wprAvgPeriod = 4;
   }
   return( wprAvgPeriod );
}
//+---------------------------------------------------------------------+
//| Perform the ATR calculation                                         |
//+---------------------------------------------------------------------+
double GetATR( int barToProcess, int barCount )
{
   double barRangeSum = 0.0;
   for (int barCounter = barToProcess; barCounter < barToProcess + barCount; barCounter++)
   {
      barRangeSum += High[barCounter] - Low[barCounter];
   }
   return( barRangeSum / 10.0 );
}
//+---------------------------------------------------------------------+
//| Find the bar after a Gap                                            |
//+---------------------------------------------------------------------+
int GetAGapBar( int barToProcess, int barCount, double minGap )
{
   int gapBar = -1;
   for (int barCounter = barToProcess; barCounter < barToProcess + barCount; barCounter++)
   {
      if ( (MathAbs(Open[barCounter] - (Close[barCounter + 1]))) >= minGap )
      {
         gapBar = barCounter;
         break;
      }
   }
   return( gapBar );
}
//+---------------------------------------------------------------------+
//| Find the bar after a a Big Move                                     |
//+---------------------------------------------------------------------+
int GetPostBigMoveBar( int barToProcess, int barCount, double moveSize )
{
   int bigMoveBar = -1;
   int bigMoveRange = 3;
   for (int barCounter = barToProcess; barCounter < (barToProcess + barCount - bigMoveRange); barCounter++)
   {
      if (MathAbs(Close[barCounter + 3] - Close[barCounter]) >= moveSize)
      {
         bigMoveBar = barCounter;
         break;
      }
   }
   return( bigMoveBar );
}
//+---------------------------------------------------------------------+
//| Post Sell Signal Alerts                                             |
//+---------------------------------------------------------------------+
void PostSellAlerts( double alertPrice )
{
   string msgText = StringConcatenate("WPR SELL signal: ",Symbol(),", Price: ",DoubleToString(Bid,Digits),
                               ", Period: ",GetPeriodText(Period()),", Stop Loss: ",DoubleToString(GetStopLoss(OP_SELL),Digits));
   if (SoundAlarm)
      Alert(msgText);
   if (EmailAlarm)
      SendMail(MailSubject, msgText);
   if (SendText)
      SendNotification(msgText);
}
//+---------------------------------------------------------------------+
//| Post Buy Signal Alerts                                              |
//+---------------------------------------------------------------------+
void PostBuyAlerts( double alertPrice )
{
   string msgText = StringConcatenate("WPR BUY signal: ",Symbol(),", Price: ",DoubleToString(Ask,Digits),
                                      ", Period: ",GetPeriodText(Period()),", Stop Loss: ",DoubleToString(GetStopLoss(OP_BUY),Digits));
   if (SoundAlarm)
      Alert(msgText);
   if (EmailAlarm)
      SendMail(MailSubject, msgText);
   if (SendText)
      SendNotification(msgText);
}
//+---------------------------------------------------------------------+
//| Place tags showing the price on the Chart                           |
//+---------------------------------------------------------------------+
void DrawPriceTag( int operation, int barToProcess )
{  // OBJ_ARROW_LEFT_PRICE OBJ_ARROW_RIGHT_PRICE
   string objName = wprPriceLabel+IntegerToString(priceLabelCount);
   priceLabelCount++;
   double labelPrice;
   if ( barToProcess == BARZERO )
      labelPrice = (operation==OP_BUY)?Ask:Bid;  // Bar 0, we have the actual Ask and Bid
   else
      labelPrice = (operation==OP_BUY)?(Open[barToProcess]+(Ask-Bid)):Open[barToProcess];  // Bars are drawn on the Bid, for past bars, synthesize Ask.
   // 
   ObjectCreate(CURRENTCHART,objName,OBJ_ARROW_LEFT_PRICE,DISPLAYWINDOWZERO,Time[barToProcess+1],labelPrice);
   ObjectSetInteger(CURRENTCHART,objName,OBJPROP_COLOR,(operation==OP_BUY)?TagBuyClr:TagSellClr);
   ObjectSetInteger(CURRENTCHART,objName,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSetInteger(CURRENTCHART,objName,OBJPROP_WIDTH,TagWidth);
   ObjectSetInteger(CURRENTCHART,objName,OBJPROP_BACK,false);
   ObjectSetInteger(CURRENTCHART,objName,OBJPROP_SELECTABLE,true);
   ObjectSetInteger(CURRENTCHART,objName,OBJPROP_HIDDEN,true);  // Not visible in the object list without clicking list all
}
//+------------------------------------------------------------------+
//| Get a Stop Loss Value for the alerts                             |
//+------------------------------------------------------------------+
double GetStopLoss( int operation )
{
   double stopLoss=0;
   if ( OP_BUY == operation )
   {
      stopLoss = Low[iLowest(Symbol(), 0, MODE_LOW, 3, 0)] - 5.0 * Point;
   }
   if ( OP_SELL == operation )
   {
      stopLoss = High[iHighest(Symbol(), 0, MODE_HIGH, 3, 0)] + 5.0 * Point;
   }
   return stopLoss;
}
//+------------------------------------------------------------------+
//| Function to provide text describing the Chart Period             |
//+------------------------------------------------------------------+
string GetPeriodText(int period)
{
   string periodText;
   switch (period)
   {
      case PERIOD_M1:
         periodText = "M1";
         break;
            // case PERIOD_M2:
            // case PERIOD_M3:
            // case PERIOD_M4:
      case PERIOD_M5:
         periodText = "M5";
         break;
            // case PERIOD_M6:
            // case PERIOD_M10:
            // case PERIOD_M12:
      case PERIOD_M15:
         periodText = "M15";
         break;
            // case PERIOD_M20:
      case PERIOD_M30:
         periodText = "M30";
         break;
      case PERIOD_H1:
         periodText = "H1";
         break;
            // case PERIOD_H2;
            // case PERIOD_H3;
      case PERIOD_H4:
         periodText = "H4";
         break;
            // case PERIOD_H6:
            // case PERIOD_H8:
            // case PERIOD_H12:
      case PERIOD_D1:
         periodText = "D1";
         break;
      case PERIOD_W1:
         periodText = "W1";
         break;
      case PERIOD_MN1:
         periodText = "MN1";
         break;
      default:
         periodText = IntegerToString(period);
         break;  // whatcha gonna do?
   }
   return(periodText);
}
//+------------------------------------------------------------------+
//| Displa Label Text on the Chart                                   |
//+------------------------------------------------------------------+
void DrawLabel( string objName, string dispText, int xPos, int yPos, int fontSize, string font, color txtClr)
{
   int windowIndex = ObjectFind(objName);
   if (windowIndex < 0 )                  // Not found, create
   {
      ObjectCreate(objName,OBJ_LABEL, DISPLAYWINDOWZERO,0,0);
      ObjectSet(objName,OBJPROP_CORNER,ANCHOR_LEFT_UPPER);   // attach the lable to the corner of the chart
      ObjectSet(objName,OBJPROP_ANCHOR,ANCHOR_LEFT_UPPER);    //  Attach the text to the corner of the lable.
      ObjectSet(objName,OBJPROP_HIDDEN,true);                 // Hidden, must press 'all' to see in object list
      ObjectSet(objName,OBJPROP_BACK,true);                  // Display on top of Background
   }
   ObjectSet(objName,OBJPROP_XDISTANCE,xPos);
   ObjectSet(objName,OBJPROP_YDISTANCE,yPos);
   ObjectSetText(objName,dispText,fontSize, font, txtClr);
}
//+------------------------------------------------------------------+
//